@@ -2,10 +2,22 @@ import pandas as pd
 import numpy as np
 import copy
 from datetime import timedelta
+from typing import Literal, List
 
 # from scipy.optimize import minimize
 from neptoon.columns import ColumnInfo
-from neptoon.corrections import Schroen2017, neutrons_to_grav_sm_desilets
+from neptoon.corrections import (
+    Schroen2017,
+    neutrons_to_total_grav_soil_moisture_desilets_etal_2010,
+)
+from neptoon.corrections.theory.neutrons_to_soil_moisture import (
+    compute_n0_koehli_etal_2021,
+)
+from neptoon.data_prep.conversions import AbsoluteHumidityCreator
+
+
+def _create_water_equiv_soc(soil_organic_carbon: float):
+    return soil_organic_carbon * 0.555
 
 
 class CalibrationConfiguration:
@@ -17,6 +29,9 @@ class CalibrationConfiguration:
         self,
         hours_of_data_around_calib: int = 6,
         converge_accuracy: float = 0.01,
+        neutron_conversion_method: Literal[
+            "desilets_etal_2010", "koehli_etal_2021"
+        ] = "desilets_etal_2010",
         calib_data_date_time_column_name: str = str(ColumnInfo.Name.DATE_TIME),
         calib_data_date_time_format: str = "%Y-%m-%d %H:%M",
         sample_depth_column: str = str(ColumnInfo.Name.CALIB_DEPTH_OF_SAMPLE),
@@ -32,22 +47,58 @@ class CalibrationConfiguration:
             ColumnInfo.Name.CALIB_SOIL_ORGANIC_CARBON
         ),
         lattice_water_column: str = str(ColumnInfo.Name.CALIB_LATTICE_WATER),
-        air_humidity_column_name: str = str(
-            ColumnInfo.Name.AIR_RELATIVE_HUMIDITY
-        ),
+        abs_air_humidity_column: str = str(ColumnInfo.Name.ABSOLUTE_HUMIDITY),
         neutron_column_name: str = str(
             ColumnInfo.Name.CORRECTED_EPI_NEUTRON_COUNT_FINAL
         ),
         air_pressure_column_name: str = str(ColumnInfo.Name.AIR_PRESSURE),
+        value_avg_lattice_water: float = 0,
+        value_avg_bulk_density: float = 0,
+        value_avg_soil_organic_carbon: float = 0,
+        koehli_method_form: Literal[
+            "Jan23_uranos",
+            "Jan23_mcnpfull",
+            "Mar12_atmprof",
+            "Mar21_mcnp_drf",
+            "Mar21_mcnp_ewin",
+            "Mar21_uranos_drf",
+            "Mar21_uranos_ewin",
+            "Mar22_mcnp_drf_Jan",
+            "Mar22_mcnp_ewin_gd",
+            "Mar22_uranos_drf_gd",
+            "Mar22_uranos_ewin_chi2",
+            "Mar22_uranos_drf_h200m",
+            "Aug08_mcnp_drf",
+            "Aug08_mcnp_ewin",
+            "Aug12_uranos_drf",
+            "Aug12_uranos_ewin",
+            "Aug13_uranos_atmprof",
+            "Aug13_uranos_atmprof2",
+        ] = "Mar21_uranos_drf",
     ):
         """
         Attributes.
 
         Parameters
         ----------
-        date_time_column_name : str, optional
-            The name of the column with date time information, by
-            default str(ColumnInfo.Name.DATE_TIME)
+        hours_of_data_around_calib : int, optional
+            Number of hours of neutron count data to include around the
+            datetime stamp for calibration. This window is used to
+            gather measurements from sensors during the calibration
+            period. Default is 6.
+        converge_accuracy : float, optional
+            The convergence threshold for when finding n0. Default is
+            0.01.
+        neutron_conversion_method : {"desilets_etal_2010",
+        "koehli_etal_2021"}, optional
+            The conversion method used to translate raw neutron counts
+            into soil moisture estimates. Options are
+            "desilets_etal_2010" or "koehli_etal_2021". Default is
+            "desilets_etal_2010".
+        calib_data_date_time_column_name : str, optional
+            The name of the column containing date‐time information for
+            each calibration day. By default, this is set to
+            str(ColumnInfo.Name.DATE_TIME).
         sample_depth_column : str, optional
             The name of the column with sample depth values (cm), by
             default str(ColumnInfo.Name.CALIB_DEPTH_OF_SAMPLE)
@@ -72,21 +123,28 @@ class CalibrationConfiguration:
         lattice_water_column : str, optional
             Name of the column with lattice water values (g/g), by
             default str(ColumnInfo.Name.CALIB_LATTICE_WATER)
-        air_humidity_column_name : str, optional
-            Name of the column with air humidity values (%), by default
-            str(ColumnInfo.Name.AIR_RELATIVE_HUMIDITY)
+        abs_air_humidity_column : str, optional
+            Name of the column with absolute air humidity values
+            (g/cm3), by default str(ColumnInfo.Name.ABSOLUTE_HUMIDITY)
         neutron_column_name : str, optional
             Name of the column with corrected neutrons in it, by default
             str(ColumnInfo.Name.CORRECTED_EPI_NEUTRON_COUNT_FINAL)
         air_pressure_column_name : str, optional
             Name of the column with air pressure vlaues in it, by
             default str(ColumnInfo.Name.AIR_PRESSURE)
+        value_avg_lattice_water: float, optional
+            The actual site average lattice water value
+        value_avg_bulk_density: float, optional
+            The actual site average dry soil bulk density
+        value_avg_soil_organic_carbon: float, optional
+            The actual site average soil organic carbon
         """
         self.hours_of_data_around_calib = hours_of_data_around_calib
         self.converge_accuracy = converge_accuracy
         self.calib_data_date_time_column_name = (
             calib_data_date_time_column_name
         )
+        self.neutron_conversion_method = neutron_conversion_method
         self.calib_data_date_time_format = calib_data_date_time_format
         self.sample_depth_column = sample_depth_column
         self.distance_column = distance_column
@@ -97,9 +155,16 @@ class CalibrationConfiguration:
         )
         self.soil_organic_carbon_column = soil_organic_carbon_column
         self.lattice_water_column = lattice_water_column
-        self.air_humidity_column_name = air_humidity_column_name
+        self.abs_air_humidity_column = abs_air_humidity_column
         self.neutron_column_name = neutron_column_name
         self.air_pressure_column_name = air_pressure_column_name
+        self.value_avg_lattice_water = value_avg_lattice_water
+        self.value_avg_bulk_density = value_avg_bulk_density
+        self.value_avg_soil_organic_carbon = value_avg_soil_organic_carbon
+        self.value_avg_soil_organic_carbon_water_equiv = (
+            _create_water_equiv_soc(value_avg_soil_organic_carbon)
+        )
+        self.koehli_method_form = koehli_method_form
 
 
 class CalibrationStation:
@@ -177,32 +242,6 @@ class SampleProfile:
 
     latest_pid = 0
 
-    __slots__ = [
-        # Input
-        "pid",  # arbitrary profile id
-        "soil_moisture_gravimetric",  # soil moisture values in g/g
-        "sm_total_grv",  # soil moisture values in g/g
-        "sm_total_vol",  # soil moisture values in g/g
-        "depth",  # depth values in cm
-        "bulk_density",  # bulk density
-        "bulk_density_mean",
-        "site_avg_bulk_density",
-        "_distance",  # distance from the CRNS in m
-        "lattice_water",  # lattice water in g/g
-        "site_avg_lattice_water",
-        "soil_organic_carbon",  # soil organic carbon in g/g
-        "site_avg_organic_carbon",
-        "calibration_day",  # the calibration day for the sample - datetime
-        # Calculated
-        "D86",  # penetration depth
-        "horizontal_weight",  # radial weight of this profile
-        "sm_total_weighted_avg_vol",  # vertically weighted average sm
-        "sm_total_weighted_avg_grv",  # vertically weighted average sm
-        "vertical_weights",
-        "rescaled_distance",
-        "data",  # DataFrame
-    ]
-
     def __init__(
         self,
         soil_moisture_gravimetric,
@@ -248,15 +287,17 @@ class SampleProfile:
         self.soil_moisture_gravimetric = np.array(soil_moisture_gravimetric)
         self.depth = np.array(depth)
         self.bulk_density = np.array(bulk_density)
-        # self.bulk_density_mean = np.array(bulk_density).mean()
         self.site_avg_bulk_density = site_avg_bulk_density
         self.calibration_day = calibration_day
         self.soil_organic_carbon = (
             np.array(soil_organic_carbon)
-            if soil_organic_carbon is None
+            if soil_organic_carbon is not None
             else np.zeros_like(soil_moisture_gravimetric)
         )
         self.site_avg_organic_carbon = site_avg_organic_carbon
+        self.site_avg_organic_carbon_water_equiv = _create_water_equiv_soc(
+            site_avg_organic_carbon
+        )
         self.lattice_water = self.lattice_water = (
             np.array(lattice_water)
             if lattice_water is not None
@@ -267,7 +308,6 @@ class SampleProfile:
         self._calculate_sm_total_vol()
         self._calculate_sm_total_grv()
 
-        # Scalar values
         self._distance = distance
         self.rescaled_distance = distance  # initialise as distance first
         self.D86 = np.nan
@@ -283,28 +323,32 @@ class SampleProfile:
         """
         Calculate total volumetric soil moisture.
         """
-        sm_total_vol = (
-            self.soil_moisture_gravimetric
-            + self.site_avg_lattice_water
-            + self.site_avg_organic_carbon * 0.555
-        ) * self.site_avg_bulk_density
-        self.sm_total_vol = sm_total_vol
+        # replace nan entries in bulk_density by the site average
+        bd = np.where(
+            np.isnan(self.bulk_density),
+            self.site_avg_bulk_density,
+            self.bulk_density,
+        )
+        self.sm_total_vol = self.soil_moisture_gravimetric * bd
 
     def _calculate_sm_total_grv(self):
         """
         Calculate total gravimetric soil moisture.
         """
-        sm_total_grv = (
-            self.soil_moisture_gravimetric
-            + self.site_avg_lattice_water
-            + self.site_avg_organic_carbon * 0.555
-        )
-        self.sm_total_grv = sm_total_grv
+
+        self.sm_total_grv = self.soil_moisture_gravimetric
 
 
 class PrepareCalibrationData:
     """
-    Prepares the calibration dataframe
+    Prepares the calibration dataframe for processing.
+
+    - ensures datetime index
+    - splits multiple days into individual data frames
+    - for each calibration day, converts data into a list of
+      SampleProfiles
+    - calculates site averages of key information (e.g., bulk density)
+    - gap fills key info with averages when missing
     """
 
     def __init__(
@@ -436,12 +480,15 @@ class PrepareCalibrationData:
         bulk_density = profile_data_frame[
             self.config.bulk_density_of_sample_column
         ]
+
         soil_moisture_gravimetric = profile_data_frame[
             self.config.soil_moisture_gravimetric_column
         ]
+
         soil_organic_carbon = profile_data_frame[
             self.config.soil_organic_carbon_column
         ]
+
         lattice_water = profile_data_frame[self.config.lattice_water_column]
         # only need one calibration datetime
         calibration_datetime = profile_data_frame[
@@ -462,36 +509,170 @@ class PrepareCalibrationData:
         )
         return soil_profile
 
+    def _create_site_avg_bulk_density(self):
+        """
+        Produces a average bulk density from provided sample data, if
+        not available uses user provided average, if thats not available
+        raises error
+
+        Returns
+        -------
+        float
+            site average bulk density
+
+        Raises
+        ------
+        ValueError
+            No bulk density provided
+        """
+        if (
+            self.config.bulk_density_of_sample_column
+            not in self.calibration_data_frame.columns
+        ):
+            message = (
+                "There appears to be no bulk density data in the calibration sample dataset. "
+                "Attempting to use the provided site average..."
+            )
+            print(message)
+            if self.config.value_avg_bulk_density is None:
+                message = (
+                    "There is no provided site average bulk density value. Please provide this "
+                    "in sensor configuration files (if using configs), SensorInformation (if using notebooks)"
+                    "or CalibrationConfiguration (if using calibration module directly.)"
+                )
+                raise ValueError(message)
+            else:
+                message = "Using the provided site average bulk density value."
+                print(message)
+
+        else:
+            message = "Calculating site average bulk density from provided sample data."
+            print(message)
+            return self.calibration_data_frame[
+                self.config.bulk_density_of_sample_column
+            ].mean()
+
+    def _create_site_avg_lattice_water(self):
+        """
+        Produces a average lattice water from provided sample data, if
+        not available uses user provided average, if thats not available
+        raises error
+
+        Returns
+        -------
+        float
+            site average lattice water
+
+        Raises
+        ------
+        ValueError
+            No lattice water provided
+        """
+        if (
+            self.config.lattice_water_column
+            not in self.calibration_data_frame.columns
+        ):
+            message = (
+                "There appears to be no lattice water data in the calibration sample dataset. "
+                "Attempting to use the provided site average..."
+            )
+            print(message)
+            if self.config.value_avg_lattice_water is None:
+                message = (
+                    "There is no provided site average lattice water value. Please provide this "
+                    "in sensor configuration files (if using configs), SensorInformation (if using notebooks)"
+                    "or CalibrationConfiguration (if using calibration module directly.)\n"
+                    "NOTE: If you don't know, use 0"
+                )
+                raise ValueError(message)
+            else:
+                message = (
+                    "Using the provided site average lattice water value."
+                )
+                print(message)
+
+        else:
+            message = "Calculating site average lattice water from provided sample data."
+            print(message)
+            return self.calibration_data_frame[
+                self.config.lattice_water_column
+            ].mean()
+
+    def _create_site_avg_soil_organic_carbon(self):
+        """
+        Produces a average soil_organic_carbon from provided sample
+        data, if not available uses user provided average, if thats not
+        available raises error
+
+        Returns
+        -------
+        float
+            site average soil_organic_carbon
+
+        Raises
+        ------
+        ValueError
+            No soil_organic_carbon provided
+        """
+        if (
+            self.config.soil_organic_carbon_column
+            not in self.calibration_data_frame.columns
+        ):
+            message = (
+                "There appears to be no soil_organic_carbon data in the calibration sample dataset. "
+                "Attempting to use the provided site average..."
+            )
+            print(message)
+            if self.config.value_avg_soil_organic_carbon is None:
+                message = (
+                    "There is no provided site average soil_organic_carbon value. Please provide this "
+                    "in sensor configuration files (if using configs), SensorInformation (if using notebooks)"
+                    "or CalibrationConfiguration (if using calibration module directly.)\n"
+                    "NOTE: If you don't know, use 0"
+                )
+                raise ValueError(message)
+            else:
+                message = "Using the provided site average soil_organic_carbon value."
+                print(message)
+
+        else:
+            message = "Calculating site average soil_organic_carbon from provided sample data."
+            print(message)
+
+            return self.calibration_data_frame[
+                self.config.soil_organic_carbon_column
+            ].mean()
+
     def prepare_calibration_data(self):
         """
         Prepares the calibration data into a list of profiles.
         """
-        site_avg_bulk_density = self.calibration_data_frame[
-            self.config.bulk_density_of_sample_column
-        ].mean()
 
-        site_avg_lattice_water = self.calibration_data_frame[
-            self.config.lattice_water_column
-        ].mean()
+        self.config.value_avg_bulk_density = (
+            self._create_site_avg_bulk_density()
+        )
 
-        site_avg_organic_carbon = self.calibration_data_frame[
-            self.config.soil_organic_carbon_column
-        ].mean()
+        self.config.value_avg_lattice_water = (
+            self._create_site_avg_lattice_water()
+        )
 
-        if np.isnan(site_avg_lattice_water):
-            site_avg_lattice_water = 0
+        self.config.value_avg_soil_organic_carbon = (
+            self.calibration_data_frame[
+                self.config.soil_organic_carbon_column
+            ].mean()
+        )
 
-        if np.isnan(site_avg_organic_carbon):
-            site_avg_organic_carbon = 0
+        if np.isnan(self.config.value_avg_soil_organic_carbon):
+            self.config.value_avg_soil_organic_carbon = 0
 
         self._create_list_of_df()
 
         for data_frame in self.list_of_data_frames:
             calibration_day_profiles = self._create_calibration_day_profiles(
                 single_day_data_frame=data_frame,
-                site_avg_bulk_density=site_avg_bulk_density,
-                site_avg_organic_carbon=site_avg_organic_carbon,
-                site_avg_lattice_water=site_avg_lattice_water,
+                site_avg_bulk_density=self.config.value_avg_bulk_density,
+                site_avg_organic_carbon=self.config.value_avg_soil_organic_carbon,
+                site_avg_lattice_water=self.config.value_avg_lattice_water,
             )
             self.list_of_profiles.extend(calibration_day_profiles)
 
@@ -510,6 +691,7 @@ class PrepareNeutronCorrectedData:
         self.data_dict = {}
 
         self._ensure_date_time_index()
+        self._ensure_abs_humidity_exists()
 
     def _ensure_date_time_index(self):
         """
@@ -521,7 +703,26 @@ class PrepareNeutronCorrectedData:
             utc=True,
         )
 
+    def _ensure_abs_humidity_exists(self):
+        """
+        Checks to see if absolute humidity exists in the data frame. If
+        it doesn't it will create it.
+        """
+        if (
+            str(ColumnInfo.Name.ABSOLUTE_HUMIDITY)
+            not in self.corrected_neutron_data_frame.columns
+        ):
+            abs_humidity_creator = AbsoluteHumidityCreator(
+                self.corrected_neutron_data_frame
+            )
+            self.corrected_neutron_data_frame = (
+                abs_humidity_creator.check_and_return_abs_hum_column()
+            )
+
     def extract_calibration_day_values(self):
+        """
+        Extracts the rows of data for each calibration day.
+        """
         calibration_indicies_dict = self._extract_calibration_day_indices(
             hours_of_data=self.config.hours_of_data_around_calib
         )
@@ -541,6 +742,7 @@ class PrepareNeutronCorrectedData:
         self.data_dict = dict_of_data
 
     def _find_nearest_calib_day_in_indicies(self, day, data_frame):
+
         day = pd.to_datetime(day)
         mask = (data_frame.index >= day - timedelta(hours=1)) & (
             data_frame.index <= day + timedelta(hours=1)
@@ -669,7 +871,7 @@ class CalibrationWeightsCalculator:
         self.calib_data_object = calib_data_object
         self.config = config
 
-        self.output_dictionary = {}
+        self.calib_metrics_dict = {}
 
     def _get_time_series_data_for_day(
         self,
@@ -677,30 +879,51 @@ class CalibrationWeightsCalculator:
     ):
         return self.time_series_data_object.data_dict[day]
 
+    @staticmethod
+    def _initial_vol_sm_estimate(profiles: List):
+        """
+        Gets an initial equal average soil moisture estimate
+
+        Parameters
+        ----------
+        profiles : List
+            List of SampleProfiles
+
+        Returns
+        -------
+        sm_estimate : float
+            Estimate of field soil moisture
+        """
+        sm_total_vol_values = [
+            np.array(profile.sm_total_vol).flatten()
+            for profile in profiles
+            if profile.sm_total_vol is not None
+        ]
+        flattened = np.concatenate(sm_total_vol_values)
+        valid_values = flattened[~np.isnan(flattened)]
+        sm_estimate = np.mean(valid_values)
+        return sm_estimate
+
     def apply_weighting_to_multiple_days(self):
+        """
+        Applies the weighting procedure to multiple calibration days if
+        present.
+        """
 
         for day in self.calib_data_object.unique_calibration_days:
 
             tmp_data = self._get_time_series_data_for_day(day)
-            day_list_of_profiles = [
-                profile
-                for profile in self.calib_data_object.list_of_profiles
-                if profile.calibration_day == day
+            profiles = [
+                p
+                for p in self.calib_data_object.list_of_profiles
+                if p.calibration_day == day
             ]
 
-            # Get initial average SM
-            sm_total_vol_values = [
-                np.array(profile.sm_total_vol).flatten()
-                for profile in day_list_of_profiles
-                if profile.sm_total_vol is not None
-            ]
-            flattened = np.concatenate(sm_total_vol_values)
-            valid_values = flattened[~np.isnan(flattened)]
-            sm_estimate = np.mean(valid_values)
-
-            # Get average air humidity and air pressure
-            average_air_humidity = tmp_data[
-                self.config.air_humidity_column_name
+            volumetric_sm_estimate = self._initial_vol_sm_estimate(
+                profiles=profiles
+            )
+            average_abs_air_humidity = tmp_data[
+                self.config.abs_air_humidity_column
             ].mean()
             average_air_pressure = tmp_data[
                 self.config.air_pressure_column_name
@@ -708,48 +931,79 @@ class CalibrationWeightsCalculator:
 
             field_average_sm_vol, field_average_sm_grav, footprint = (
                 self.calculate_weighted_sm_average(
-                    day_list_of_profiles=day_list_of_profiles,
-                    initial_sm_estimate=sm_estimate,
-                    average_air_humidity=average_air_humidity,
+                    profiles=profiles,
+                    initial_volumetric_sm_estimate=volumetric_sm_estimate,
+                    average_abs_air_humidity=average_abs_air_humidity,
                     average_air_pressure=average_air_pressure,
                 )
             )
 
-            info_dictionary = {
+            output = {
                 "field_average_soil_moisture_volumetric": field_average_sm_vol,
                 "field_average_soil_moisture_gravimetric": field_average_sm_grav,
                 "horizontal_footprint_radius_in_meters": footprint,
+                "absolute_air_humidity": average_abs_air_humidity,
+                "atmospheric_pressure": average_air_pressure,
             }
 
-            self.output_dictionary[day] = info_dictionary
+            self.calib_metrics_dict[day] = output
 
     def calculate_weighted_sm_average(
         self,
-        day_list_of_profiles,
-        initial_sm_estimate: float,
-        average_air_humidity: float,
+        profiles: List,
+        initial_volumetric_sm_estimate: float,
+        average_abs_air_humidity: float,
         average_air_pressure: float,
     ):
-        """_summary_
+        """
+        Calculates the field average soil moisture value according to
+        Schrön et al., 2017. Weighting is calculated on volumetric soil
+        moisture volumes (as described in the paper), but a weighted
+        gravimetric value is given for calibration in neptoon.
 
         Parameters
         ----------
-        day_list_of_profiles : _type_
-            _description_
+        profiles : List[Profile]
+            A list of soil‐profile objects collected on the same day.
+            Each Profile must have:
+              - `.rescaled_distance` (rescaled distance from sensor, in
+                m)
+              - `.site_avg_bulk_density` (bulk density, in g/cm3)
+              - `.depth` (array of depths, in cm)
+              - `.sm_total_vol` (array of volumetric‐moisture values,
+                cm3/cm3)
+              - `.sm_total_grv` (array of gravimetric‐moisture values,
+                g/g)
         initial_sm_estimate : float
-            _description_
-        average_air_humidity : float
-            _description_
+            Initial soil moisture estimate (usually equal average)
+        average_abs_air_humidity : float
+            Average absolute air humidity
         average_air_pressure : float
-            _description_
+            Air pressure average during calibration period (hPa)
 
         Returns
         -------
-        _type_
-            _description_
+        field_average_sm_volumetric : float
+            Converged volumetric soil moisture (cm3/cm3).
+        field_average_sm_gravimetric : float
+            Corresponding converged gravimetric soil moisture (g/g).
+        footprint_m : float
+            Estimated radius (m) of the footprint of the sensor.
+
+        Notes
+        -----
+        - Convergence is checked via `abs((new_volumetric_estimate -
+          old_estimate) / old_estimate) <
+          self.config.converge_accuracy`.
+        - Uses `Schroen2017.rescale_distance`,
+          `Schroen2017.calculate_measurement_depth`,
+          `Schroen2017.vertical_weighting`,
+          `Schroen2017.horizontal_weighting`, and
+          `Schroen2017.calculate_footprint_radius` at each iteration.
+
         """
 
-        sm_estimate = copy.deepcopy(initial_sm_estimate)
+        volumetric_sm_estimate = copy.deepcopy(initial_volumetric_sm_estimate)
         accuracy = 1
         field_average_sm_volumetric = None
         field_average_sm_gravimetric = None
@@ -759,48 +1013,49 @@ class CalibrationWeightsCalculator:
             profile_sm_averages_gravimetric = []
             profiles_horizontal_weights = []
 
-            for profile in day_list_of_profiles:
+            for p in profiles:
 
-                profile.rescaled_distance = Schroen2017.rescale_distance(
-                    distance=profile.rescaled_distance,
-                    pressure=average_air_pressure,
-                    soil_moisture=sm_estimate,
+                p.rescaled_distance = Schroen2017.rescale_distance(
+                    distance_from_sensor=p.rescaled_distance,
+                    atmospheric_pressure=average_air_pressure,
+                    volumetric_soil_moisture=volumetric_sm_estimate,
                 )
 
-                profile.D86 = Schroen2017.calculate_measurement_depth(
-                    distance=profile.rescaled_distance,
-                    bulk_density=profile.site_avg_bulk_density,
-                    soil_moisture=sm_estimate,
+                p.D86 = Schroen2017.calculate_measurement_depth(
+                    distance=p.rescaled_distance,
+                    bulk_density=p.site_avg_bulk_density,
+                    volumetric_soil_moisture=volumetric_sm_estimate,
                 )
 
-                profile.vertical_weights = Schroen2017.vertical_weighting(
-                    profile.depth,
-                    bulk_density=profile.site_avg_bulk_density,
-                    soil_moisture=sm_estimate,
+                p.vertical_weights = Schroen2017.vertical_weighting(
+                    depth=p.depth,
+                    distance=p.rescaled_distance,
+                    bulk_density=p.site_avg_bulk_density,
+                    volumetric_soil_moisture=volumetric_sm_estimate,
                 )
 
                 # Calculate weighted sm average
-                profile.sm_total_weighted_avg_vol = np.average(
-                    profile.sm_total_vol, weights=profile.vertical_weights
+                p.sm_total_weighted_avg_vol = np.average(
+                    p.sm_total_vol, weights=p.vertical_weights
                 )
-                profile.sm_total_weighted_avg_grv = np.average(
-                    profile.sm_total_grv, weights=profile.vertical_weights
+                p.sm_total_weighted_avg_grv = np.average(
+                    p.sm_total_grv, weights=p.vertical_weights
                 )
 
-                profile.horizontal_weight = Schroen2017.horizontal_weighting(
-                    distance=profile.rescaled_distance,
-                    soil_moisture=profile.sm_total_weighted_avg_vol,
-                    air_humidity=average_air_humidity,
+                p.horizontal_weight = Schroen2017.horizontal_weighting(
+                    distance=p.rescaled_distance,
+                    volumetric_soil_moisture=p.sm_total_weighted_avg_vol,
+                    abs_air_humidity=average_abs_air_humidity,
                 )
 
                 # create a list of average sm and horizontal weights
                 profile_sm_averages_volumetric.append(
-                    profile.sm_total_weighted_avg_vol
+                    p.sm_total_weighted_avg_vol
                 )
                 profile_sm_averages_gravimetric.append(
-                    profile.sm_total_weighted_avg_grv
+                    p.sm_total_weighted_avg_grv
                 )
-                profiles_horizontal_weights.append(profile.horizontal_weight)
+                profiles_horizontal_weights.append(p.horizontal_weight)
 
             # mask out nan values from list
             profile_sm_averages_volumetric = np.ma.MaskedArray(
@@ -829,19 +1084,22 @@ class CalibrationWeightsCalculator:
 
             # check convergence accuracy
             accuracy = abs(
-                (field_average_sm_volumetric - sm_estimate) / sm_estimate
+                (field_average_sm_volumetric - volumetric_sm_estimate)
+                / volumetric_sm_estimate
             )
             if accuracy > self.config.converge_accuracy:
 
-                sm_estimate = copy.deepcopy(field_average_sm_volumetric)
+                volumetric_sm_estimate = copy.deepcopy(
+                    field_average_sm_volumetric
+                )
                 profile_sm_averages_volumetric = []
                 profile_sm_averages_gravimetric = []
                 profiles_horizontal_weights = []
 
         footprint_m = Schroen2017.calculate_footprint_radius(
-            soil_moisture=field_average_sm_volumetric,
-            air_humidity=average_air_humidity,
-            pressure=average_air_pressure,
+            volumetric_soil_moisture=field_average_sm_volumetric,
+            abs_air_humidity=average_abs_air_humidity,
+            atmospheric_pressure=average_air_pressure,
         )
 
         return (
@@ -851,7 +1109,16 @@ class CalibrationWeightsCalculator:
         )
 
     def return_output_dict_as_dataframe(self):
-        df = pd.DataFrame.from_dict(self.output_dictionary, orient="index")
+        """
+        Returns the dictionary of information created for each
+        calibration day during processing as a pandas DataFrame
+
+        Returns
+        -------
+        pd.DataFrame
+            DataFrame with information created during processing.
+        """
+        df = pd.DataFrame.from_dict(self.calib_metrics_dict, orient="index")
         df = df.reset_index()
         df = df.rename(
             columns={
@@ -863,50 +1130,44 @@ class CalibrationWeightsCalculator:
         )
         return df
 
-    def find_optimal_N0(
-        self,
-    ):
-        df = self.return_output_dict_as_dataframe()
-        for index, row in df.iterrows():
-            calib_day = pd.to_datetime(row["calibration_day"])
-            calib_data_frame = self.time_series_data_object.data_dict[
-                calib_day
-            ]
-            gravimetric_sm_on_day = row[
-                "field_average_soil_moisture_gravimetric"
-            ]
-
-            N0_optimal, absolute_error = (
-                self.find_optimal_N0_single_day_iteration_style(
-                    gravimetric_sm_on_day=gravimetric_sm_on_day,
-                    calib_data_frame_subset=calib_data_frame,
-                )
-            )
-
-            # Update the original dictionary
-            self.output_dictionary[calib_day]["optimal_N0"] = N0_optimal
-            self.output_dictionary[calib_day][
-                "absolute_error"
-            ] = absolute_error
-        df = self.return_output_dict_as_dataframe()
-        average_n0 = df["optimal_N0"].mean()
-        return average_n0
-
-    def find_optimal_N0_single_day_iteration_style(
+    def _find_optimal_N0_single_day_desilets_etal_2010(
         self,
         gravimetric_sm_on_day,
-        calib_data_frame_subset,
+        neutron_mean,
+        lattice_water=0,
+        water_equiv_soil_organic_carbon=0,
     ):
-        neutron_mean = calib_data_frame_subset[
-            self.config.neutron_column_name
-        ].mean()
+        """
+        Finds optimal N0 number when using desilets et al., 2010 method
+
+        Parameters
+        ----------
+        gravimetric_sm_on_day : float
+            gravimetric soil moisture (weighted)
+        neutron_mean : float
+            average (corrected) neutron count
+
+        Returns
+        -------
+        float
+            N0
+        """
+
         n0_range = pd.Series(range(int(neutron_mean), int(neutron_mean * 2.5)))
+        gravimetric_sm_on_day_total = (
+            gravimetric_sm_on_day
+            + lattice_water
+            + water_equiv_soil_organic_carbon
+        )
 
         def calculate_sm_and_error(n0):
-            sm_prediction = neutrons_to_grav_sm_desilets(
-                neutrons=neutron_mean, n0=n0
+            sm_prediction = (
+                neutrons_to_total_grav_soil_moisture_desilets_etal_2010(
+                    neutrons=neutron_mean,
+                    n0=n0,
+                )
             )
-            absolute_error = abs(sm_prediction - gravimetric_sm_on_day)
+            absolute_error = abs(sm_prediction - gravimetric_sm_on_day_total)
             return pd.Series(
                 {
                     "N0": n0,
@@ -921,26 +1182,93 @@ class CalibrationWeightsCalculator:
         minimum_error = results_df.loc[min_error_idx, "absolute_error"]
         return n0_optimal, minimum_error
 
-    def find_optimal_N0_single_day_minimise_style(
+    def _find_optimal_n0_single_day_koehli_etal_2021(
         self,
-        gravimetric_sm_on_day,
-        average_neutrons_on_day,
-        N0_initial=1000,
-        N0_bounds=(400, 4000),
+        gravimetric_sm_on_day: float,
+        neutron_mean: int,
+        abs_air_humidity: float,
+        lattice_water: float,
+        water_equiv_soil_organic_carbon: float,
+        koehli_method_form: str,
     ):
-        """TODO"""
-        # result = minimize(
-        #     self.find_N0_error_function,
-        #     N0_initial,
-        #     args=(
-        #         gravimetric_sm_on_day,
-        #         self.time_series_data_object.corrected_neutron_data_frame[
-        #             self.neutron_column_name
-        #         ],
-        #     ),
-        #     bounds=[N0_bounds],
-        # )
-        # N0_optimal = result.x[0]
-        # rmse = result["fun"]
-        # return N0_optimal, rmse
-        pass
+        """
+        Finds optimal N0 number when using Koehli etal method
+
+        Parameters
+        ----------
+        gravimetric_sm_on_day : float
+            Average gravimetic water on calibration day)
+        neutron_mean : float | int
+            Mean corrected neutron count
+        abs_air_humidity : float
+            Absolute air humidity
+        lattice_water : float
+            Lattice water content of soil
+        water_equiv_soil_organic_carbon : float
+            water equivelant of soil organic carbon
+        koehli_method_form: str
+            The specific method form of Koehli method
+
+        Returns
+        -------
+        Tuple
+            The N0 calibration term and absolute error (dummy nan value)
+        """
+        n0 = compute_n0_koehli_etal_2021(
+            gravimetric_sm=gravimetric_sm_on_day,
+            neutron_count=neutron_mean,
+            air_humidity=abs_air_humidity,
+            lattice_water=lattice_water,
+            water_equiv_soil_organic_carbon=water_equiv_soil_organic_carbon,
+            koehli_method_form=koehli_method_form,
+        )
+        return n0, "nan"
+
+    def find_optimal_N0(
+        self,
+    ):
+        """
+        Finds the optimal N0 number for the site using the weighted
+        field average soil mositure.
+
+        Returns
+        -------
+        average_n0
+            The optimal n0 across all the supplied calibration days.
+        """
+
+        for day, metrics in self.calib_metrics_dict.items():
+            neutron_mean = self.time_series_data_object.data_dict[day][
+                self.config.neutron_column_name
+            ].mean()
+            grav_sm = metrics["field_average_soil_moisture_gravimetric"]
+
+            if self.config.neutron_conversion_method == "desilets_etal_2010":
+                n0_opt, abs_error = (
+                    self._find_optimal_N0_single_day_desilets_etal_2010(
+                        gravimetric_sm_on_day=grav_sm,
+                        neutron_mean=neutron_mean,
+                        lattice_water=self.config.value_avg_lattice_water,
+                        water_equiv_soil_organic_carbon=self.config.value_avg_soil_organic_carbon_water_equiv,
+                    )
+                )
+
+            elif self.config.neutron_conversion_method == "koehli_etal_2021":
+                n0_opt, abs_error = (
+                    self._find_optimal_n0_single_day_koehli_etal_2021(
+                        gravimetric_sm_on_day=grav_sm,
+                        neutron_mean=neutron_mean,
+                        abs_air_humidity=metrics["absolute_air_humidity"],
+                        lattice_water=self.config.value_avg_lattice_water,
+                        water_equiv_soil_organic_carbon=self.config.value_avg_soil_organic_carbon_water_equiv,
+                        koehli_method_form=self.config.koehli_method_form,
+                    )
+                )
+
+            metrics.update({"optimal_N0": n0_opt, "absolute_error": abs_error})
+
+        return float(
+            np.mean(
+                [m["optimal_N0"] for m in self.calib_metrics_dict.values()]
+            )
+        )
