@@ -73,7 +73,7 @@ class NMDBDataAttacher:
     def configure(
         self,
         station: str,
-        reference_value: int = None,
+        reference_value: int | None = None,
         resolution="60",
         nmdb_table="revori",
     ):
@@ -131,9 +131,6 @@ class NMDBDataAttacher:
             str(ColumnInfo.Name.REFERENCE_INCOMING_NEUTRON_VALUE)
         ] = self.config.reference_value
 
-        self.data_frame[str(ColumnInfo.Name.NMDB_REFERENCE_STATION)] = (
-            self.config.station
-        )
         self.data_frame[
             str(ColumnInfo.Name.REFERENCE_MONITOR_CUTOFF_RIGIDITY)
         ] = NMDB_CUTOFF_RIGIDITIES[self.config.station]
@@ -573,7 +570,7 @@ class DataFetcher:
 
         nmdb_form = "wget"
         url = (
-            f"http://nest.nmdb.eu/draw_graph.php?{nmdb_form}=1"
+            f"https://www.nmdb.eu/nest/draw_graph.php?{nmdb_form}=1"
             f"&stations[]={self.config.station}"
             f"&tabchoice={self.config.nmdb_table}"
             f"&dtype=corr_for_efficiency"
