@@ -1,6 +1,7 @@
 import pandas as pd
 import numpy as np
 from typing import Literal, Union, Optional
+import datetime
 from pathlib import Path
 from neptoon.external.nmdb_data_collection import (
     NMDBDataAttacher,
@@ -15,7 +16,6 @@ from neptoon.corrections.factory.build_corrections import (
     CorrectionType,
     CorrectionTheory,
     CorrectNeutrons,
-    NeutronUncertaintyCalculator,
 )
 from neptoon.config.configuration_input import SensorInfo
 from neptoon.products.estimate_sm import NeutronsToSM
@@ -26,9 +26,15 @@ from neptoon.quality_control import (
     QualityAssessmentFlagBuilder,
     DataQualityAssessor,
 )
+from neptoon.utils import (
+    validate_timestamp_index,
+    find_temporal_resolution_seconds,
+)
 from neptoon.visulisation.figures_handler import FigureHandler
 from neptoon.io.save import SaveAndArchiveOutputs
 from neptoon.data_prep.smoothing import SmoothData
+from neptoon.data_prep.conversions import AbsoluteHumidityCreator
+from neptoon.data_prep import TimeStampAggregator, TimeStampAligner
 from neptoon.columns import ColumnInfo
 from neptoon.logging import get_logger
 from magazine import Magazine
@@ -54,11 +60,11 @@ class CRNSDataHub:
     def __init__(
         self,
         crns_data_frame: pd.DataFrame,
-        flags_data_frame: pd.DataFrame = None,
-        sensor_info: SensorInfo = None,
-        quality_assessor: DataQualityAssessor = None,
+        flags_data_frame: pd.DataFrame | None = None,
+        sensor_info: SensorInfo | None = None,
+        quality_assessor: DataQualityAssessor | None = None,
         validation: bool = True,
-        calibration_samples_data: pd.DataFrame = None,
+        calibration_samples_data: pd.DataFrame | None = None,
     ):
         """
         Inputs to the CRNSDataHub.
@@ -201,7 +207,7 @@ class CRNSDataHub:
         new_column_name=str(ColumnInfo.Name.INCOMING_NEUTRON_INTENSITY),
         resolution="60",
         nmdb_table="revori",
-        reference_value=None,
+        reference_value: int | None = None,
     ):
         """
         Utilises the NMDBDataAttacher class to attach NMDB incoming
@@ -246,7 +252,7 @@ class CRNSDataHub:
 
     def add_quality_flags(
         self,
-        custom_flags: QualityAssessmentFlagBuilder = None,
+        custom_flags: QualityAssessmentFlagBuilder | None = None,
         add_check=None,
     ):
         """
@@ -297,7 +303,6 @@ class CRNSDataHub:
             A string representing a default version of flagging, by
             default None
         """
-
         self.quality_assessor.apply_quality_assessment()
 
         self.flags_data_frame = self.quality_assessor.return_flags_data_frame(
@@ -311,8 +316,8 @@ class CRNSDataHub:
 
     def select_correction(
         self,
-        correction_type: CorrectionType = "empty",
-        correction_theory: CorrectionTheory = None,
+        correction_type: CorrectionType | str = "empty",
+        correction_theory: CorrectionTheory | None = None,
     ):
         """
         Method to select corrections to be applied to data.
@@ -349,17 +354,6 @@ class CRNSDataHub:
         )
         self.crns_data_frame = corrector.correct_neutrons()
 
-    def create_neutron_uncertainty_bounds(self):
-        """
-        Create uncertainty bounds for statistical uncertainty of neutron
-        count rates.
-        """
-
-        uncertainty = NeutronUncertaintyCalculator(
-            data_frame=self.crns_data_frame
-        )
-        self.crns_data_frame = uncertainty.add_neutron_uncertainty_columns()
-
     @Magazine.reporting(topic="Data Preparation")
     def smooth_data(
         self,
@@ -368,6 +362,7 @@ class CRNSDataHub:
             "rolling_mean", "savitsky_golay"
         ] = "rolling_mean",
         window: Optional[Union[int, str]] = 12,
+        min_proportion_good_data: float = 0.7,
         poly_order: int = 4,
         auto_update_final_col: bool = True,
     ):
@@ -396,17 +391,17 @@ class CRNSDataHub:
         Data smoothing was done on {column_to_smooth}. This was done
         using {smooth_method} with a window of {window}.
         """
-        series_to_smooth = pd.Series(self.crns_data_frame[column_to_smooth])
+        print(f"Smoothing data with a smoothing window of {window}")
         smoother = SmoothData(
-            data=series_to_smooth,
+            data=self.crns_data_frame,
             column_to_smooth=column_to_smooth,
             smooth_method=smooth_method,
             window=window,
+            min_proportion_good_data=min_proportion_good_data,
             poly_order=poly_order,
             auto_update_final_col=auto_update_final_col,
         )
-        col_name = smoother.create_new_column_name()
-        self.crns_data_frame[col_name] = smoother.apply_smoothing()
+        self.crns_data_frame = smoother.apply_smoothing()
 
     @Magazine.reporting(topic="Calibration")
     def calibrate_station(
@@ -430,7 +425,7 @@ class CRNSDataHub:
         Report
         ------
         Calibration was undertaken. The N0 number was calculated as
-        {n0}. From the samples, the average dry soil bulk density is
+        {n0}, using the {config.neutron_conversion_method} method. From the samples, the average dry soil bulk density is
         {avg_dry_soil_bulk_density}, the average soil organic carbon is
         {avg_soil_organic_carbon}, and the average lattice water content
         is {avg_lattice_water}.
@@ -473,13 +468,88 @@ class CRNSDataHub:
         self.sensor_info.avg_soil_organic_carbon = avg_soil_organic_carbon
         print(f"N0 number was calculated as {n0}")
 
+    def align_time_stamps(
+        self,
+        align_method: str = "time",
+    ):
+        """
+        Aligns timestamps to occur on the hour. E.g., 01:00 not 01:05.
+
+        Uses the TimeStampAligner class.
+
+        Parameters
+        ----------
+        method : str, optional
+            method to use for shifting, defaults to shifting to nearest
+            hour, by default "time"
+        """
+        print("Aligning timestamps to regular intervals...")
+        timestamp_aligner = TimeStampAligner(self.crns_data_frame)
+        timestamp_aligner.align_timestamps(
+            method=align_method,
+        )
+        self.crns_data_frame = timestamp_aligner.return_dataframe()
+
+    def aggregate_data_frame(
+        self,
+        output_resolution: str,
+        max_na_fraction: float = 0.3,
+        aggregate_method: str = "bagg",
+    ):
+        """
+        Aggregate a crns data frame to a new resolution.
+
+        Parameters
+        ----------
+        output_resolution : str
+            Desired output resolution (e.g., '1h' or '1day')
+        max_na_fraction : float, optional
+            fraction of acceptable nan values in aggregation period, by
+            default 0.3
+        aggregate_method : str, optional
+            _description_, by default "bagg"
+        """
+        print(f"Aggregating data to {output_resolution} resolution")
+        timestamp_aggregator = TimeStampAggregator(
+            data_frame=self.crns_data_frame,
+            output_resolution=output_resolution,
+            max_na_fraction=max_na_fraction,
+        )
+        timestamp_aggregator.aggregate_data(
+            method=aggregate_method,
+        )
+        self.crns_data_frame = timestamp_aggregator.return_dataframe()
+
     @Magazine.reporting(topic="Soil Moisture")
     def produce_soil_moisture_estimates(
         self,
-        n0: float = None,
-        dry_soil_bulk_density: float = None,
-        lattice_water: float = None,
-        soil_organic_carbon: float = None,
+        n0: float | None = None,
+        conversion_theory: Literal[
+            "desilets_etal_2010", "koehli_eta_2021"
+        ] = "desilets_etal_2010",
+        dry_soil_bulk_density: float | None = None,
+        lattice_water: float | None = None,
+        soil_organic_carbon: float | None = None,
+        koehli_method_form: Literal[
+            "Jan23_uranos",
+            "Jan23_mcnpfull",
+            "Mar12_atmprof",
+            "Mar21_mcnp_drf",
+            "Mar21_mcnp_ewin",
+            "Mar21_uranos_drf",
+            "Mar21_uranos_ewin",
+            "Mar22_mcnp_drf_Jan",
+            "Mar22_mcnp_ewin_gd",
+            "Mar22_uranos_drf_gd",
+            "Mar22_uranos_ewin_chi2",
+            "Mar22_uranos_drf_h200m",
+            "Aug08_mcnp_drf",
+            "Aug08_mcnp_ewin",
+            "Aug12_uranos_drf",
+            "Aug12_uranos_ewin",
+            "Aug13_uranos_atmprof",
+            "Aug13_uranos_atmprof2",
+        ] = "Mar21_uranos_drf",
     ):
         """
         Produces SM estimates with the NeutronsToSM class. If values for
@@ -506,6 +576,7 @@ class CRNSDataHub:
         soil organic carbon content of
         {default_params[soil_organic_carbon]}
         """
+        print("Producing soil moisture estimates.")
         # Create attributes for NeutronsToSM
         default_params = {
             "n0": self.sensor_info.N0,
@@ -518,6 +589,8 @@ class CRNSDataHub:
             "dry_soil_bulk_density": dry_soil_bulk_density,
             "lattice_water": lattice_water,
             "soil_organic_carbon": soil_organic_carbon,
+            "conversion_theory": conversion_theory,
+            "koehli_method_form": koehli_method_form,
         }
         params = {k: v for k, v in provided_params.items() if v is not None}
         default_params.update(params)
@@ -564,10 +637,36 @@ class CRNSDataHub:
                 core_logger.info(message)
                 continue
             elif value is None:
-                # TODO add skip here
+                core_logger.debug(f"Skipping None value for {key}")
+            elif isinstance(value, (datetime.datetime, datetime.date)):
+                core_logger.debug(
+                    f"Skipping datetime value for {key}: {value}"
+                )
                 continue
             else:
-                self.crns_data_frame[key] = value
+                try:
+                    numeric_value = pd.to_numeric(value)
+                    self.crns_data_frame[key] = numeric_value
+                except (ValueError, TypeError):
+                    # Skip non-numeric values
+                    core_logger.debug(
+                        f"Skipping non-numeric value for {key}: {value}"
+                    )
+                    continue
+
+    def prepare_additional_columns(self):
+        """
+        Prepares and adds additional columns required for processing.
+
+        Such as:
+           - absolute humidity
+        """
+        abs_hum_creator = AbsoluteHumidityCreator(
+            data_frame=self.crns_data_frame
+        )
+        self.crns_data_frame = (
+            abs_hum_creator.check_and_return_abs_hum_column()
+        )
 
     def create_figures(
         self,
